#ifndef __USB_DESCRIPTORS_C
#define __USB_DESCRIPTORS_C

#include "./USB/usb.h"
#include "./USB/usb_function_hid.h"
#include "./USB/usb_function_cdc.h"

/* Device Descriptor */
ROM USB_DEVICE_DESCRIPTOR device_dsc = {
    0x12, // Size of this descriptor in bytes
    USB_DESCRIPTOR_DEVICE, // DEVICE descriptor type
    0x0200, // USB Spec Release Number in BCD format
    0xEF, // Class Code "MISC_DEVICE" (ex: uses IAD descriptor)
    0x02, // Subclass code
    0x01, // Protocol code
    USB_EP0_BUFF_SIZE, // Max packet size for EP0, see usb_config.h
    MY_VID, // Vendor ID
    MY_PID, // Product ID: Custom HID device demo
    0x0001, // Device release number in BCD format
    0x01, // Manufacturer string index
    0x02, // Product string index
    0x03, // Device serial number string index
    0x01 // Number of possible configurations
};

/* Configuration 1 Descriptor */
ROM BYTE configDescriptor1[] = {
    /* Configuration Descriptor */
    0x09, //sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
    USB_DESCRIPTOR_CONFIGURATION, // CONFIGURATION descriptor type
    0x6B, 0x00, // Total length of data for this cfg // 0x29+0x08+0x3a
    USB_MAX_NUM_INT, // Number of interfaces in this cfg
    1, // Index value of this configuration
    0, // Configuration string index
    _DEFAULT | _SELF, // Attributes, see usb_device.h
    250, // Max power consumption (2x mA)

    /* Interface Descriptor */
    0x09, //sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE, // INTERFACE descriptor type
    HID_INTF_ID, // Interface Number
    0, // Alternate Setting Number
    2, // Number of endpoints in this intf
    HID_INTF, // Class code
    0, // Subclass code
    0, // Protocol code
    0, // Interface string index

    /* HID Class-Specific Descriptor */
    0x09, //sizeof(USB_HID_DSC)+3,    // Size of this descriptor in bytes
    DSC_HID, // HID descriptor type
    0x11, 0x01, // HID Spec Release Number in BCD format (1.11)
    0x00, // Country Code (0x00 for Not supported)
    HID_NUM_OF_DSC, // Number of class descriptors, see usbcfg.h
    DSC_RPT, // Report descriptor type
    HID_RPT01_SIZE, 0x00, //sizeof(hid_rpt01),      // Size of the report descriptor

    /* Endpoint Descriptor */
    0x07, /*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT, //Endpoint Descriptor
    HID_EP | _EP_IN, //EndpointAddress
    _INTERRUPT, //Attributes
    0x40, 0x00, //size
    10, //Interval

    /* Endpoint Descriptor */
    0x07, /*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT, //Endpoint Descriptor
    HID_EP | _EP_OUT, //EndpointAddress
    _INTERRUPT, //Attributes
    0x40, 0x00, //size
    10, //Interval

    /* Interface Association Descriptor: CDC Function 1*/
    0x08, //sizeof(USB_IAD_DSC), // Size of this descriptor in bytes
    0x0B, // Interface assocication descriptor type
    CDC_COMM_INTF_ID, // The first associated interface
    2, // Number of contiguous associated interface
    COMM_INTF, // bInterfaceClass of the first interface
    ABSTRACT_CONTROL_MODEL, // bInterfaceSubclass of the first interface
    V25TER, // bInterfaceProtocol of the first interface
    0, // Interface string index

    /* Interface Descriptor */
    9, //sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE, // INTERFACE descriptor type
    CDC_COMM_INTF_ID, // Interface Number
    0, // Alternate Setting Number
    1, // Number of endpoints in this intf
    COMM_INTF, // Class code
    ABSTRACT_CONTROL_MODEL, // Subclass code
    V25TER, // Protocol code
    0, // Interface string index

    /* CDC Class-Specific Descriptors */
    sizeof (USB_CDC_HEADER_FN_DSC),
    CS_INTERFACE,
    DSC_FN_HEADER,
    0x10, 0x01,

    sizeof (USB_CDC_ACM_FN_DSC),
    CS_INTERFACE,
    DSC_FN_ACM,
    USB_CDC_ACM_FN_DSC_VAL,

    sizeof (USB_CDC_UNION_FN_DSC),
    CS_INTERFACE,
    DSC_FN_UNION,
    CDC_COMM_INTF_ID,
    CDC_DATA_INTF_ID,

    sizeof (USB_CDC_CALL_MGT_FN_DSC),
    CS_INTERFACE,
    DSC_FN_CALL_MGT,
    0x00,
    CDC_DATA_INTF_ID,

    /* Endpoint Descriptor */
    //sizeof(USB_EP_DSC),DSC_EP,_EP02_IN,_INT,CDC_INT_EP_SIZE,0x02,
    0x07, /*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT, //Endpoint Descriptor
    CDC_COMM_EP | _EP_IN, //EndpointAddress
    _INTERRUPT, //Attributes
    0x0A, 0x00, //size
    0x01, //Interval

    /* Interface Descriptor */
    9, //sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE, // INTERFACE descriptor type
    CDC_DATA_INTF_ID, // Interface Number
    0, // Alternate Setting Number
    2, // Number of endpoints in this intf
    DATA_INTF, // Class code
    0, // Subclass code
    NO_PROTOCOL, // Protocol code
    0, // Interface string index

    /* Endpoint Descriptor */
    //sizeof(USB_EP_DSC),DSC_EP,_EP03_OUT,_BULK,CDC_BULK_OUT_EP_SIZE,0x00,
    0x07, /*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT, //Endpoint Descriptor
    CDC_DATA_EP | _EP_OUT, //EndpointAddress
    _BULK, //Attributes
    0x40, 0x00, //size
    0x00, //Interval

    /* Endpoint Descriptor */
    //sizeof(USB_EP_DSC),DSC_EP,_EP03_IN,_BULK,CDC_BULK_IN_EP_SIZE,0x00
    0x07, /*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT, //Endpoint Descriptor
    CDC_DATA_EP | _EP_IN, //EndpointAddress
    _BULK, //Attributes
    0x40, 0x00, //size
    0x00 //Interval
};

//Language code string descriptor

ROM struct
{
    BYTE bLength;
    BYTE bDscType;
    WORD string[1];
}

sd000 = {
    sizeof (sd000), USB_DESCRIPTOR_STRING,
    {
        0x0409
    }
};

//Manufacturer string descriptor

ROM struct
{
    BYTE bLength;
    BYTE bDscType;
    WORD string[25];
}

sd001 = {
    sizeof (sd001), USB_DESCRIPTOR_STRING,
    {
        'M', 'i', 'c', 'r', 'o', 'c', 'h', 'i', 'p', ' ',
        'T', 'e', 'c', 'h', 'n', 'o', 'l', 'o', 'g', 'y', ' ', 'I', 'n', 'c', '.'
    }
};

//Product string descriptor

ROM struct
{
    BYTE bLength;
    BYTE bDscType;
    WORD string[32];
}

sd002 = {
    sizeof (sd002), USB_DESCRIPTOR_STRING,
    {
        'C', 'o', 'm', 'p', 'o', 's', 'i', 't', 'e', ' ', 'H', 'I', 'D', ' ', '+', ' ',
        'C', 'D', 'C', ',', ' ', 'A', 'P', 'P', '-', 'E', 'S', 'S', '1', '4', '-', '1'
    }
};

ROM struct
{
    BYTE bLength;
    BYTE bDscType;
    WORD string[10];
}

sd003 = {
    sizeof (sd003), USB_DESCRIPTOR_STRING,
    {
        'N', 'o', ' ', 'S', 'u', 'p', 'p', 'o', 'r', 't'
    }
};

//Class specific descriptor - HID

ROM struct
{
    BYTE report[HID_RPT01_SIZE];
} hid_rpt01 = {
    {
        0x06, 0x00, 0xFF, // Usage Page = 0xFF00 (Vendor Defined Page 1)
        0x09, 0x01, // Usage (Vendor Usage 1)
        0xA1, 0x01, // Collection (Application)
        0x19, 0x01, // Usage Minimum
        0x29, 0x40, // Usage Maximum 	//64 input usages total (0x01 to 0x40)
        0x15, 0x01, // Logical Minimum (data bytes in the report may have minimum value = 0x00)
        0x25, 0x40, // Logical Maximum (data bytes in the report may have maximum value = 0x00FF = unsigned 255)
        0x75, 0x08, // Report Size: 8-bit field size
        0x95, 0x40, // Report Count: Make sixty-four 8-bit fields (the next time the parser hits an "Input", "Output", or "Feature" item)
        0x81, 0x00, // Input (Data, Array, Abs): Instantiates input packet fields based on the above report size, count, logical min/max, and usage.
        0x19, 0x01, // Usage Minimum
        0x29, 0x40, // Usage Maximum 	//64 output usages total (0x01 to 0x40)
        0x91, 0x00, // Output (Data, Array, Abs): Instantiates output packet fields.  Uses same report size and count as "Input" fields, since nothing new/different was specified to the parser since the "Input" item.
        0xC0
    } // End Collection
};

//Array of configuration descriptors
ROM BYTE *ROM USB_CD_Ptr[] = {
    (ROM BYTE * ROM) & configDescriptor1
};

//Array of string descriptors
ROM BYTE *ROM USB_SD_Ptr[] = {
    (ROM BYTE * ROM) & sd000,
    (ROM BYTE * ROM) & sd001,
    (ROM BYTE * ROM) & sd002,
    (ROM BYTE * ROM) & sd003
};

#endif
