#include <xc.h>
#include <stdio.h>

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( FRC * 4 ) / 1
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 , Disable SOSC
#pragma config POSCMOD = NONE, FNOSC = FRCPLL , OSCIOFNC = IO
#pragma config FWDTEN = OFF , WINDIS = OFF , FWPSA = PR128 , WDTPS = PS32768
#pragma config ICS = PGx2

#define SystemFrequency     ( 8000000L * 4 ) / 1 / 2
#define Timer1Tick          ( SystemFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 300
#define UART1Period         ( ( SystemFrequency / UART1Baud ) / 16 ) - 1
#define UART1Baud           38400L

unsigned int VR1Value = 0, PAD1Value = 0, PAD2Value = 0;
volatile unsigned int ADC1Done = 0;

unsigned char UARTString1[ 200 ];
volatile unsigned char UART_IsReceived = 0;
volatile unsigned char UART_ReceiveBuffer = 0;
volatile unsigned int isOneSec = 0, isLCDUpdate = 0;
unsigned char Hours = 0, Mins = 0, Secs = 0;

#define i2cDataBufferSize 0x20
#define i2cSlaveAddr 0x40
volatile unsigned char i2cStageLoop = 0;
volatile unsigned int i2cBufferSize = 0;
volatile unsigned int i2cBufOffset = 0;
volatile unsigned char * i2cBufBasePtr;
volatile unsigned char i2cDataBuffer[ i2cDataBufferSize ];

volatile unsigned char LEDblinkStatus = 1;
volatile unsigned char ADCChScanIndex = 0;

/* i2c Command Assign  */
/* SlaveAddr AddrH AddrL Data */

/* i2c Data Buffer Assign     */
/* 0x00  VRValue Low Byte     */
/* 0x01  VRValue High Byte    */
/* 0x02  PAD1Value Low Byte   */
/* 0x03  PAD1Value High Byte  */
/* 0x04  PAD2Value Low Byte   */
/* 0x05  PAD2Value High Byte  */
/* 0x08  SW1 Status           */
/* 0x09  SW2 Status           */
/* 0x10  LED1 Status          */
/* 0x11  LED2 Status          */
/* 0x12  LED3 Status          */
/* 0x13  LED4 Status          */

void InitializeSystem(void);
void UART_PutRAMString(unsigned char * String);

void __attribute__((interrupt, no_auto_psv)) _T1Interrupt(void)
{
    static unsigned int T1Tick1 = 0;
    static unsigned int T1Tick2 = 0;
    static unsigned int T1Tick3 = 0;

    unsigned int i = 0;

    IFS0bits.T1IF = 0;

    switch (++ADCChScanIndex)
    {
        case 1:
            CTMUCON1Lbits.IDISSEN = 0x01; // Discharge Internal Cap.
            for (i = 0; i < 2; i++);
            CTMUCON1Lbits.IDISSEN = 0x00;

            AD1CHSbits.CH0NA = 0x00; // AVSS
            AD1CHSbits.CH0SA = 5; // VR1 (AN5)
            break;

        case 2:
            AD1CHSbits.CH0NA = 0x00; // AVSS
            AD1CHSbits.CH0SA = 13; // PAD1 (AN13)

            CTMUCON1Lbits.IDISSEN = 0x01; // Discharge Internal Cap.
            for (i = 0; i < 2; i++);
            CTMUCON1Lbits.IDISSEN = 0x00;

            TRISAbits.TRISA2 = 0x01; // Enable Analog Mode AN13/RA2
            ANSAbits.ANSA2 = 0x01;

            CTMUCON1Lbits.CTMUEN = 1; // Enable CTMU
            CTMUCON1Lbits.IRNG = 0x03; // 100 x base current
            CTMUCON1Lbits.ITRIM = 0x1f;
            break;

        case 3:
            AD1CHSbits.CH0NA = 0x00; // AVSS
            AD1CHSbits.CH0SA = 14; // PAD2 (AN14)

            CTMUCON1Lbits.IDISSEN = 0x01; // Discharge Internal Cap.
            for (i = 0; i < 2; i++);
            CTMUCON1Lbits.IDISSEN = 0x00;

            TRISAbits.TRISA3 = 0x01; // Enable Analog Mode AN14/RA3
            ANSAbits.ANSA3 = 0x01;

            CTMUCON1Lbits.CTMUEN = 1; // Enable CTMU
            CTMUCON1Lbits.IRNG = 0x03; // 100 x base current
            CTMUCON1Lbits.ITRIM = 0x1f;
            break;
    }

    CTMUCON1Hbits.EDG1STAT = 1;
    for (i = 0; i < 20; i++);
    CTMUCON1Hbits.EDG1STAT = 0;

    AD1CON1bits.SAMP = 0x00;

    if (++T1Tick1 > 29)
    {
        T1Tick1 = 0;
        isLCDUpdate = 1;
    }

    if (++T1Tick2 > 74)
    {
        T1Tick2 = 0;

        if (LEDblinkStatus)
        {
            // 改變LED狀態。
            i2cDataBuffer[0x10] = ~i2cDataBuffer[0x10] & 0x01;
            i2cDataBuffer[0x11] = ~i2cDataBuffer[0x11] & 0x01;
            i2cDataBuffer[0x12] = ~i2cDataBuffer[0x12] & 0x01;
            i2cDataBuffer[0x13] = ~i2cDataBuffer[0x13] & 0x01;
            LATAbits.LATA0 = i2cDataBuffer[0x10];
            LATAbits.LATA1 = i2cDataBuffer[0x11];
            LATBbits.LATB0 = i2cDataBuffer[0x12];
            LATBbits.LATB1 = i2cDataBuffer[0x13];
        }
    }

    if (++T1Tick3 > 299)
    {
        T1Tick3 = 0;
        isOneSec = 1;
    }
}

void __attribute__((interrupt, no_auto_psv)) _ADC1Interrupt(void)
{
    // 讀取ADC轉換之數值。

    static unsigned int PAD1AvgValue = 0, PAD2AvgValue = 0;

    CTMUCON1Lbits.CTMUEN = 0; // Disable CTMU

    switch (ADCChScanIndex)
    {
        case 1:
            VR1Value = (unsigned int) ADC1BUF0;
            break;

        case 2:
            PAD1Value = (unsigned int) ADC1BUF0;
            PAD1AvgValue = (PAD1AvgValue >> 1) + (PAD1AvgValue >> 2) + (PAD1AvgValue >> 3); // 1/2 + 1/4 + 1/8 = 7/8
            PAD1AvgValue += (PAD1Value >> 3);
            LATAbits.LATA2 = 0x00; // Discharge AN13/RA2
            TRISAbits.TRISA2 = 0x00;
            break;

        case 3:
            PAD2Value = (unsigned int) ADC1BUF0;
            PAD2AvgValue = (PAD2AvgValue >> 1) + (PAD2AvgValue >> 2) + (PAD2AvgValue >> 3); // 1/2 + 1/4 + 1/8 = 7/8
            PAD2AvgValue += (PAD2Value >> 3);
            LATAbits.LATA3 = 0x00; // Discharge AN14/RA3
            TRISAbits.TRISA3 = 0x00;
            ADCChScanIndex = 0;
            break;
    }

    // 清除ADC中斷旗標
    IFS0bits.AD1IF = 0;

    AD1CON1bits.SAMP = 0x01;

    i2cDataBuffer[0x00] = VR1Value & 0xff;
    i2cDataBuffer[0x01] = (VR1Value >> 8) & 0xff;
    i2cDataBuffer[0x02] = PAD1AvgValue & 0xff;
    i2cDataBuffer[0x03] = (PAD1AvgValue >> 8) & 0xff;
    i2cDataBuffer[0x04] = PAD2AvgValue & 0xff;
    i2cDataBuffer[0x05] = (PAD2AvgValue >> 8) & 0xff;
}

void __attribute__((interrupt, no_auto_psv)) _U1RXInterrupt(void)
{
    // 清除UART2 Rereived的中斷旗標。
    IFS0bits.U1RXIF = 0;

    // Stop LED toggle Function
    LEDblinkStatus = 0;

    UART_ReceiveBuffer = U1RXREG;

    UART_IsReceived = 1;
}

void __attribute__((interrupt, auto_psv)) _MSSP1Interrupt(void)
{
    unsigned char Temporary = 0;

    IFS1bits.SSP1IF = 0;

    // Stop LED toggle Function
    LEDblinkStatus = 0;

    if (SSP1STATbits.P)
    {
        i2cStageLoop = 0;

        LATAbits.LATA0 = i2cDataBuffer[0x10];
        LATAbits.LATA1 = i2cDataBuffer[0x11];
        LATBbits.LATB0 = i2cDataBuffer[0x12];
        LATBbits.LATB1 = i2cDataBuffer[0x13];

        // Disable i2c Module
        SSP1CON1bits.SSPEN = 0x00;
        Nop();
        Nop();
        // Enable i2c Module
        SSP1CON1bits.SSPEN = 0x01;
        return;
    }

    // If Received Slave Address ?
    if (!SSP1STATbits.D_A)
    {
        Temporary = SSP1BUF;

        if (SSP1STATbits.R_W)
            SSP1BUF = *(i2cBufBasePtr + i2cBufOffset);
        i2cStageLoop = 0;
    }

    else
    {
        if (SSP1STATbits.R_W)
        {
            // if Received ACK
            if (!SSP1CON2bits.ACKSTAT)
                SSP1BUF = *(i2cBufBasePtr + i2cBufOffset);
        }

        else
        {
            Temporary = SSP1BUF;

            switch (++i2cStageLoop)
            {
                case 1:
                    i2cBufOffset = ((unsigned int) Temporary) << 8;
                    break;

                case 2:
                    i2cBufOffset += (unsigned int) Temporary;
                    break;

                case 3:
                    i2cStageLoop = 2; // Provide Overflow

                    *(i2cBufBasePtr + i2cBufOffset) = Temporary;
                    break;
            }
        }
    }
    SSP1CON1bits.CKP = 1;
}

void __attribute__((interrupt, auto_psv)) _DefaultInterrupt(void)
{
    while (1);
}

int main(void)
{
    // Divide by 1
    CLKDIVbits.RCDIV = 0;

    // CPU Clock = PB Clock / 1
    CLKDIVbits.DOZEN = 0;

    InitializeSystem();

    while (1)
    {
        i2cDataBuffer[0x08] = !PORTBbits.RB14;
        i2cDataBuffer[0x09] = !PORTBbits.RB15;

        // UART收到資料 ?
        if (UART_IsReceived)
        {
            UART_IsReceived = 0;

            switch (UART_ReceiveBuffer)
            {
                case 'A':
                case 'a':
                    i2cDataBuffer[0x10] = ~i2cDataBuffer[0x10] & 0x01;
                    LATAbits.LATA0 = i2cDataBuffer[0x10];
                    break;
                case 'B':
                case 'b':
                    i2cDataBuffer[0x11] = ~i2cDataBuffer[0x11] & 0x01;
                    LATAbits.LATA1 = i2cDataBuffer[0x11];
                    break;
                case 'C':
                case 'c':
                    i2cDataBuffer[0x12] = ~i2cDataBuffer[0x12] & 0x01;
                    LATBbits.LATB0 = i2cDataBuffer[0x12];
                    break;
                case 'D':
                case 'd':
                    i2cDataBuffer[0x13] = ~i2cDataBuffer[0x13] & 0x01;
                    LATBbits.LATB1 = i2cDataBuffer[0x13];
                    break;
            }
        }

        if (isOneSec)
        {
            isOneSec = 0;

            if (++Secs > 59)
            {
                Secs = 0;
                if (++Mins > 59)
                {
                    Mins = 0;
                    if (++Hours > 23)
                        Hours = 0;
                }
            }
        }

        if (isLCDUpdate)
        {
            isLCDUpdate = 0;

            //sprintf((char *) UARTString1, "Total Running Time : %2d:%2d:%2d , AD Value = %4d , BT4 = %1d , BT3 = %1d \r",
            //        Hours, Mins, Secs, VR1Value, !PORTBbits.RB15, !PORTBbits.RB14);
            sprintf((char *) UARTString1, "Total Running Time : %2d:%2d:%2d , AD Value = %4d , PAD1 Value = %4d , PAD2 Value = %4d , BT4 = %1d , BT3 = %1d \r",
                    Hours, Mins, Secs,
                    ((int) i2cDataBuffer[0x01] << 8) + i2cDataBuffer[0x00],
                    ((int) i2cDataBuffer[0x03] << 8) + i2cDataBuffer[0x02],
                    ((int) i2cDataBuffer[0x05] << 8) + i2cDataBuffer[0x04],
                    i2cDataBuffer[0x09], i2cDataBuffer[0x08]);

            // 使用自訂函數將字串透過UART輸出。
            UART_PutRAMString(UARTString1);
        }
    }
    while (1);
}

void InitializeSystem(void)
{
    unsigned int i = 0;

    TRISAbits.TRISA0 = 0; // LED1
    TRISAbits.TRISA1 = 0; // LED2
    TRISBbits.TRISB0 = 0; // LED3
    TRISBbits.TRISB1 = 0; // LED4

    ANSBbits.ANSB14 = 0;
    ANSBbits.ANSB15 = 0;

    LATAbits.LATA0 = 0;
    LATAbits.LATA1 = 1;
    LATBbits.LATB0 = 0;
    LATBbits.LATB1 = 1;

    // 啟用Timer1。
    PR1 = Timer1Tick;
    T1CONbits.TON = 1;
    T1CONbits.TCS = 0;
    T1CONbits.TGATE = 0;
    T1CONbits.TCKPS = 0x03;
    // 啟用Timer1 Interrupt。
    IFS0bits.T1IF = 0;
    IPC0bits.T1IP = 0x04;
    IEC0bits.T1IE = 1;

    // 啟用ADC。
    AD1CON1bits.ADON = 0x01; // Enable ADC
    AD1CON1bits.FORM = 0x00; // Integer
    AD1CON1bits.ASAM = 0x00; // Mamual Sample
    AD1CON1bits.SAMP = 0x01;
    AD1CON1bits.SSRC = 0x00; // Manual
    AD1CON1bits.MODE12 = 0x01; // 12 Bit Mode
    AD1CON2bits.PVCFG = 0x00; // VR+ = AVDD
    AD1CON2bits.NVCFG0 = 0x00; // VR- = AVSS
    AD1CON2bits.CSCNA = 0x00; // Disable SCAN Function
    AD1CON2bits.SMPI = 0x00; // 1 Conversion pre Interrupts
    AD1CON2bits.BUFM = 0x00; // Disable Ping Pong Buffer
    AD1CON2bits.ALTS = 0x00; // Diable Alternate Input
    AD1CON3bits.ADRC = 0x01; // Conversion Clock use Tcy
    AD1CON3bits.ADCS = 0x01; // 64 Tcy
    AD1CON3bits.SAMC = 0x0c; // 31 TAD

    ANSBbits.ANSB5 = 0x01; // Enable Analog Mode
    ANSAbits.ANSA2 = 0x01; // Enable Analog Mode
    ANSAbits.ANSA3 = 0x01; // Enable Analog Mode

    AD1CHSbits.CH0NA = 0x00; // AVSS
    AD1CHSbits.CH0SA = 0x05; // VR1 (AN5)

    // 啟用ADC1 Interrupt。
    IFS0bits.AD1IF = 0;
    IPC3bits.AD1IP = 0x04;
    IEC0bits.AD1IE = 1;

    // 啟用UART1。
    ANSBbits.ANSB2 = 0;
    U1MODEbits.UARTEN = 0x01;
    U1MODEbits.UEN = 0x00; // UxTX, UxRX Only, No flow control pins
    U1MODEbits.PDSEL = 0x00; // 8-Bit Data, No Parity
    U1MODEbits.STSEL = 0x00; // 1 Stop Bit
    U1MODEbits.BRGH = 0x00; // Low Baud Rate
    U1MODEbits.RXINV = 0x00;
    U1STAbits.TXINV = 0x00;
    U1STAbits.UTXEN = 0x01;
    U1STAbits.UTXISEL0 = 0x00; // Interrupt when a character is transferred to the Transmit Shift Register
    U1STAbits.UTXISEL1 = 0x00; //
    U1STAbits.URXISEL0 = 0x00; // Interrupt is set when any character is received
    U1STAbits.URXISEL1 = 0x00; //
    U1BRG = UART1Period;
    // 啟用UART1 Interrupt。
    IFS0bits.U1RXIF = 0;
    IPC2bits.U1RXIP = 0x04;
    IEC0bits.U1RXIE = 1;

    // 啟用I2C1。
    SSP1CON1bits.SSPM = 0x06; // Slave 7-Bits Mode
    SSP1STATbits.SMP = 0x00; // Enabled for High-Speed (400KHz)
    SSP1CON2bits.SEN = 1; // Enable Clock Stretch
    SSP1ADDbits.I2CADD = i2cSlaveAddr & 0xfe;
    SSP1CON2bits.GCEN = 0x00;
    SSP1MSK = 0xff;
    SSP1CON3bits.PCIE = 0x01; // Enable Stop Condition Interrupt
    SSP1CON1bits.SSPEN = 0x01;

    ANSBbits.ANSB8 = 0;
    ANSBbits.ANSB9 = 0;

    // I2C Slave Buffer Init
    for (i = 0; i < i2cDataBufferSize; i++)
        i2cDataBuffer[ i ] = 0;

    i2cBufBasePtr = (volatile unsigned char *) &i2cDataBuffer[0];
    i2cBufferSize = i2cDataBufferSize;

    // LED Status Set
    i2cDataBuffer[0x10] = 0;
    i2cDataBuffer[0x11] = 1;
    i2cDataBuffer[0x12] = 0;
    i2cDataBuffer[0x13] = 1;
    LATAbits.LATA0 = i2cDataBuffer[0x10];
    LATAbits.LATA1 = i2cDataBuffer[0x11];
    LATBbits.LATB0 = i2cDataBuffer[0x12];
    LATBbits.LATB1 = i2cDataBuffer[0x13];

    IFS1bits.SSP1IF = 0;
    IPC4bits.SSP1IP = 0x04;
    IEC1bits.SSP1IE = 1;
}

void UART_PutRAMString(unsigned char * String)
{
    while (*String != 0x00)
    {
        while (U1STAbits.UTXBF);
        U1TXREG = *String++;
    }
}
